@@ -17,10 +17,10 @@
 - (NSString *)pathToColumn:(int)column;
 - (void)showNewDir;
 - (NSString *)makeNJBPathString:(NSString *)path;
-- (void)addItem:(MyItem *)item toDir:(NSArray *)pathArray;
 - (void)addCreateDirToQueue:(NSString *)dirName;
 - (void)createDir:(NSString *)dirName;
 - (void)copyFromNJBDirectory:(Directory *)dir toPath:(NSString *)path;
+- (void)deleteDir:(Directory *)dir fromParent:(Directory *)parent;
 @end
 
 // todo: implement drag and drop into the njbBrowser
@@ -206,43 +206,44 @@
 	
 	NSArray *destDirPathComponents;
 	NSString *destDir = @"";
-	if ([theNJB isProtocol3Device])
-	{	
-		destDir = [NSString stringWithString:[dataFile fullPath]];
-		destDir = [destDir substringFromIndex:[path length]+1];
-		destDir = [NSString stringWithFormat:@"%@%@", [njbBrowser containingPath], destDir];
-		destDir = [destDir stringByDeletingLastPathComponent];
-		// destDir now doesn't end in a /
+	
+	destDir = [NSString stringWithString:[dataFile fullPath]];
+	destDir = [destDir substringFromIndex:[path length]+1];
+	destDir = [NSString stringWithFormat:@"%@%@", [njbBrowser containingPath], destDir];
+	destDir = [destDir stringByDeletingLastPathComponent];
+	// destDir now doesn't end in a /
 		
-		// so we need to remove just the first object from here
-		destDirPathComponents = [destDir pathComponents];
-		if ([destDirPathComponents count] > 0)
-		{
-			NSMutableArray *mutableComponents = [NSMutableArray arrayWithArray:destDirPathComponents];
-			[mutableComponents removeObjectAtIndex:0];
-			destDirPathComponents = [NSArray arrayWithArray:mutableComponents];
-		}
-	}
-	else
+	// so we need to remove just the first object from here
+	destDirPathComponents = [destDir pathComponents];
+	if ([destDirPathComponents count] > 0)
 	{
-		destDirPathComponents = [NSArray arrayWithObjects:nil];
+		NSMutableArray *mutableComponents = [NSMutableArray arrayWithArray:destDirPathComponents];
+		[mutableComponents removeObjectAtIndex:0];
+		destDirPathComponents = [NSArray arrayWithArray:mutableComponents];
 	}
 	
 	NJBQueueItem *copyFile = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadFile:toFolder:) withObject:dataFile];
 	[copyFile setObject2:[self makeNJBPathString:destDir]];
 	[copyFile setStatus:STATUS_UPLOADING_FILE];
+		
+	NJBQueueItem *getUpdatedFileList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(downloadFileList)];
+	[getUpdatedFileList setDisplayStatus:NO];
 	
-	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addItem:toDir:) withObject:dataFile withRunInMainThread:YES];
-	[addToList setObject2:destDirPathComponents];
-	[addToList setDisplayStatus:NO];
-	
-	[copyFile cancelItemIfFail:addToList];
+	NJBQueueItem *updateTable = [[NJBQueueItem alloc] initWithTarget:self
+																											withSelector:@selector(showNewDir)
+																												withObject:nil
+																							 withRunInMainThread:YES];
+	[updateTable setDisplayStatus:NO];
+		
+	[copyFile cancelItemIfFail:getUpdatedFileList];
+	[getUpdatedFileList cancelItemIfFail:updateTable];
 	
 	NSString *description = [NSString stringWithFormat:NSLocalizedString(@"Uploading file '%@'", nil), [dataFile filename]];
-	[self addToQueue:copyFile withSubItems:[NSArray arrayWithObject:addToList] withDescription:description];
+	[self addToQueue:copyFile withSubItems:[NSArray arrayWithObjects:getUpdatedFileList, updateTable, nil] withDescription:description];
 	
 	[copyFile release];
-	[addToList release];
+	[getUpdatedFileList release];
+	[updateTable release];
 }
 
 /* delete the item from the NJB
@@ -255,46 +256,35 @@
 /* delete the item from the NJB
 */
 - (void)deleteFromNJB:(MyItem *)item fromParent:(Directory *)parentDir
-{
-	NJBQueueItem *removeFromParent = nil;
-	if (parentDir != nil)
+{	
+	if (parentDir == nil)
 	{
-		removeFromParent = [[NJBQueueItem alloc] initWithTarget:parentDir withSelector:@selector(removeItem:)
-																								 withObject:item];
-		[removeFromParent setRunInMainThread:YES];
-		[removeFromParent setDisplayStatus:NO];
+		NSLog(@"parentDir == nil in deleteFromNJB:fromParent! Not updating cache so expect problems\n");
 	}
 	
-	NJBQueueItem *reloadNJBDirs = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(reloadNJBDirs)];
-	[reloadNJBDirs setRunInMainThread:YES];
-	[reloadNJBDirs setDisplayStatus:NO];
+	NJBQueueItem *getUpdatedFileList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(downloadFileList)];
+	[getUpdatedFileList setDisplayStatus:NO];
 	
-	// a folder not created (so just made by putting a file in it) will have ID 0 and cannot be deleted
-	// so just remove them from the parent (if we can)
-	if ([item isMemberOfClass:[Directory class]] && [item itemID] == 0 && parentDir != nil)
-	{
-		[self addToQueue:removeFromParent withSubItems:[NSArray arrayWithObjects:reloadNJBDirs, nil]];
-	}
-	else if ([item itemID] != 0) // we can't delete anything with 0 ID (if still left from above)
-	{
-		NJBQueueItem *deleteFile = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(deleteFile:)
-																											 withObject:([NSNumber numberWithInt:[item itemID]])];
-		[deleteFile setStatus:STATUS_DELETING_FILE];
+	NJBQueueItem *updateTable = [[NJBQueueItem alloc] initWithTarget:self
+																											withSelector:@selector(showNewDir)
+																												withObject:nil
+																							 withRunInMainThread:YES];
+	[updateTable setDisplayStatus:NO];
 	
-		[deleteFile cancelItemIfFail:reloadNJBDirs];
-		if (removeFromParent != nil)
-			[deleteFile cancelItemIfFail:removeFromParent];
+	NJBQueueItem *deleteFile = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(deleteFile:fromDir:)
+																											 withObject:item];
+	[deleteFile setObject2:parentDir];
+	[deleteFile setStatus:STATUS_DELETING_FILE];
 	
-		NSString *description = [NSString stringWithFormat:NSLocalizedString(@"Deleting file '%@'", nil), [item description]];
-		if (removeFromParent != nil)
-			[self addToQueue:deleteFile withSubItems:[NSArray arrayWithObjects:removeFromParent, reloadNJBDirs, nil] withDescription:description];
-		else
-			[self addToQueue:deleteFile withSubItems:[NSArray arrayWithObjects:reloadNJBDirs, nil] withDescription:description];
+	[deleteFile cancelItemIfFail:getUpdatedFileList];
+	[getUpdatedFileList cancelItemIfFail:updateTable];
+	
+	NSString *description = [NSString stringWithFormat:NSLocalizedString(@"Deleting file '%@'", nil), [item description]];
+	[self addToQueue:deleteFile withSubItems:[NSArray arrayWithObjects:getUpdatedFileList, updateTable, nil] withDescription:description];
 		
-		[deleteFile release];
-	}
-	[removeFromParent release];
-	[reloadNJBDirs release];
+	[deleteFile release];
+	[getUpdatedFileList release];
+	[updateTable release];
 }
 
 - (void)fileSystemBrowserFileClick:(NSString *)path
@@ -391,26 +381,34 @@
 - (IBAction)newFolder:(id)sender
 {
 	NSString *newFolderName = [newFolderText stringValue];
-	NSString *parentDir = [self makeNJBPathString:[njbBrowser containingPath]];
-	
 	Directory *dir = [[Directory alloc] initWithName:newFolderName];
+	NSString *parentDir = [self makeNJBPathString:[njbBrowser containingPath]];
+	NJBQueueItem *createFolder = nil;
 	
-	NJBQueueItem *createFolder = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(createFolder:inDir:)
-																												 withObject:dir];
+	createFolder = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(createFolder:inDir:)
+																													 withObject:dir];
 	[createFolder setObject2:parentDir];
 	[createFolder setStatus:STATUS_CREATING_FOLDER];
 	
-	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addItem:toDir:) withObject:dir withRunInMainThread:YES];
-	[addToList setObject2:[njbBrowser containingPathComponents]];
-	[addToList setDisplayStatus:NO];
+	NJBQueueItem *getUpdatedFileList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(downloadFileList)];
+	[getUpdatedFileList setDisplayStatus:NO];
+	
+	NJBQueueItem *updateTable = [[NJBQueueItem alloc] initWithTarget:self
+																											withSelector:@selector(showNewDir)
+																												withObject:nil
+																							 withRunInMainThread:YES];
+	[updateTable setDisplayStatus:NO];
 		
-	[createFolder cancelItemIfFail:addToList];
+	[createFolder cancelItemIfFail:getUpdatedFileList];
+	[getUpdatedFileList cancelItemIfFail:updateTable];
 	
 	NSString *description = [NSString stringWithFormat:NSLocalizedString(@"Creating folder '%@'", nil), newFolderName];
-	[self addToQueue:createFolder withSubItems:[NSArray arrayWithObjects:addToList, nil] withDescription:description];
+	[self addToQueue:createFolder withSubItems:[NSArray arrayWithObjects:getUpdatedFileList, updateTable, nil] withDescription:description];
 		
 	[createFolder release];
-	[addToList release];
+	[getUpdatedFileList release];
+	[updateTable release];
+	[dir release];
 }
 
 - (void)NJBDisconnected:(NSNotification *)note
@@ -441,13 +439,6 @@
 	return [NSString stringWithString:temp];
 }
 
-- (void)addItem:(MyItem *)item toDir:(NSArray *)pathArray
-{
-	[baseDir addItem:item toDir:pathArray];
-	
-	[self reloadNJBDirs];
-}
-
 // override this - we do it ourselves on clicks
 - (void)onFirstResponderChange:(NSResponder *)aResponder
 {}
@@ -488,20 +479,27 @@
 	{
 		itemToDelete = [containingDir itemWithDescription:[currentCell stringValue]];
 		if ([itemToDelete isMemberOfClass:[Directory class]])
-		{
-			Directory *dir = (Directory *)itemToDelete;
-			NSArray *subMembers = [dir recursiveMembers];
-			NSEnumerator *subEnumerator = [subMembers objectEnumerator];
-			MyItem *subItemToDelete;
-			while (subItemToDelete = [subEnumerator nextObject])
-			{
-				[self deleteFromNJB:subItemToDelete];
-			}
-		}
-		[self deleteFromNJB:itemToDelete fromParent:containingDir];
+			[self deleteDir:(Directory *)itemToDelete fromParent:containingDir];
+		else
+			[self deleteFromNJB:itemToDelete fromParent:containingDir];
 	}
 }
 
+// deletes the directory and all files/directories contained in it
+- (void)deleteDir:(Directory *)dir fromParent:(Directory *)parent
+{
+	NSEnumerator *objectEnumerator = [dir objectEnumerator];
+	MyItem *toDelete = nil;
+	while (toDelete = [objectEnumerator nextObject])
+	{
+		if ([toDelete isMemberOfClass:[Directory class]])
+			[self deleteDir:(Directory *)toDelete fromParent:dir];
+		else
+			[self deleteFromNJB:toDelete fromParent:dir];
+	}
+	[self deleteFromNJB:dir fromParent:parent];
+}
+
 - (void)reloadNJBDirs
 {
 	[njbBrowser reloadColumn:[njbBrowser lastColumn]];
@@ -551,7 +549,7 @@
 {
 	if (![theNJB isConnected])
 		return NO;
-	if (![theNJB isProtocol3Device])
+	if (![theNJB isProtocol3Device] && ![theNJB isMTPDevice])
 		return NO;
 	if ([[newFolderText stringValue] length] > 0)
 		return YES;
