@@ -148,8 +148,8 @@
 	if ([track genre])
 		[genreCombo setStringValue:[track genre]];
 	[yearField setStringValue:[NSString stringWithFormat:@"%d", [track year]]];
-	if ([track codecString])
-		[codecField setStringValue:[track codecString]];
+	if ([track fileTypeString])
+		[codecField setStringValue:[track fileTypeString]];
 	if ([track lengthString])
 		[lengthField setStringValue:[track lengthString]];
 	
