@@ -215,19 +215,16 @@ StatusDisplayer *statusDisplayerGlobal;
 	if (debug)
 		NJB_Set_Debug(debug);
 	
+	[self setStatus:STATUS_CONNECTING];
+	
 	if (NJB_Discover(njbs, 0, &n) == -1)
 	{
-		[self setStatus:STATUS_NO_NJB];
+		//[self setStatus:STATUS_NO_NJB];
 		errorString = NSLocalizedString(@"Could not discover any jukeboxes", nil);
 	}
 	else
 	{
-		if (n == 0)
-		{
-			[self setStatus:STATUS_NO_NJB];
-			errorString = NSLocalizedString(@"Could not locate any jukeboxes", nil);
-		}
-		else
+		if (n > 0)
 		{
 			njb = &njbs[0];
 	
