@@ -397,9 +397,14 @@
     parsedV1 = YES;         
     if ([v1Tag tagPresent]==YES)
     {
-            genreName = [NSArray arrayWithObject:[[dataDictionary objectForKey:@"genreIndexes"] objectForKey:[[NSNumber numberWithInt:[v1Tag getGenre]] stringValue]]];
-            if (genreName == NULL) genreName = [NSArray arrayWithObject:@"Unknown"];
-            return genreName;
+			// bug fix by RAL 20060130: check for an invalid genre
+			id object = [[dataDictionary objectForKey:@"genreIndexes"] objectForKey:[[NSNumber numberWithInt:[v1Tag getGenre]] stringValue]];
+			if (object != nil)
+				genreName = [NSArray arrayWithObject:object];
+			else
+				genreName = [NSArray arrayWithObject:@"Unknown"];
+        
+			return genreName;
     }
     return genreName;
 }
