@@ -84,6 +84,7 @@ typedef struct LIBMTP_playlist_struct LIBMTP_playlist_t; /**< @see LIBMTP_playli
 typedef struct LIBMTP_album_struct LIBMTP_album_t; /**< @see LIBMTP_album_struct */
 typedef struct LIBMTP_folder_struct LIBMTP_folder_t; /**< @see LIBMTP_folder_t */
 typedef struct LIBMTP_object_struct LIBMTP_object_t; /**< @see LIBMTP_object_t */
+typedef struct LIBMTP_filesampledata_struct LIBMTP_filesampledata_t; /**< @see LIBMTP_filesample_t */
 
 /**
  * The callback type definition. Notice that a progress percentage ratio
@@ -224,17 +225,15 @@ struct LIBMTP_folder_struct {
 };
 
 /**
- * LIBMTP Object Structure
+ * LIBMTP Object RepresentativeSampleData Structure
  */
-struct LIBMTP_object_struct {
-  uint32_t id;
-  uint32_t parent;
-  uint32_t type;
-  uint32_t size;
-  char *name;
-  void *data;
-  LIBMTP_object_t *sibling;
-  LIBMTP_object_t *child;
+struct LIBMTP_filesampledata_struct {
+  uint32_t width; /**< Width of sample if it is an image */
+  uint32_t height; /**< Height of sample if it is an image */
+  uint32_t duration; /**< Duration in milliseconds if it is audio */
+  LIBMTP_filetype_t filetype; /**< Filetype used for the sample */
+  uint64_t size; /**< Size of sample data in bytes */
+  char *data; /**< Sample data */
 };
 
 /** @} */
@@ -299,6 +298,14 @@ int LIBMTP_Send_File_From_File(LIBMTP_mtpdevice_t *, char const * const,
 int LIBMTP_Send_File_From_File_Descriptor(LIBMTP_mtpdevice_t *, int const,
 	                LIBMTP_file_t * const, LIBMTP_progressfunc_t const,
 			void const * const, uint32_t const);
+LIBMTP_filesampledata_t *LIBMTP_new_filesampledata_t(void);
+void LIBMTP_destroy_filesampledata_t(LIBMTP_filesampledata_t *);
+int LIBMTP_Get_Representative_Sample_Format(LIBMTP_mtpdevice_t *,
+                        LIBMTP_filetype_t const,
+                        LIBMTP_filesampledata_t **);
+int LIBMTP_Send_Representative_Sample(LIBMTP_mtpdevice_t *, uint32_t const,
+                          LIBMTP_filesampledata_t *);
+
 /**
  * @}
  * @defgroup tracks The track management API.
@@ -363,47 +370,16 @@ LIBMTP_album_t *LIBMTP_Get_Album_List(LIBMTP_mtpdevice_t *);
 LIBMTP_album_t *LIBMTP_Get_Album(LIBMTP_mtpdevice_t *, uint32_t const);
 int LIBMTP_Create_New_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t * const, uint32_t const);
 int LIBMTP_Update_Album(LIBMTP_mtpdevice_t *, LIBMTP_album_t const * const);
-int LIBMTP_Send_Album_Art(LIBMTP_mtpdevice_t *, uint32_t const, uint8_t * const, uint32_t const);
 
 /**
  * @}
  * @defgroup objects The object management API.
  * @{
  */
-char *LIBMTP_Get_String_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t);
-uint32_t LIBMTP_Get_U32_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint32_t);
-uint16_t LIBMTP_Get_U16_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint16_t);
-uint8_t LIBMTP_Get_U8_From_Object(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint8_t);
-int LIBMTP_Set_Object_String(LIBMTP_mtpdevice_t *,const uint32_t,const uint16_t,
-                             char const * const);
-int LIBMTP_Set_Object_U32(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint32_t);
-int LIBMTP_Set_Object_U16(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint16_t);
-int LIBMTP_Set_Object_U8(LIBMTP_mtpdevice_t *,const uint32_t, const uint16_t, const uint8_t);
-int LIBMTP_Get_Object_References(LIBMTP_mtpdevice_t *, const uint32_t, uint32_t **, uint32_t *);
-int LIBMTP_Set_Object_References(LIBMTP_mtpdevice_t *, const uint32_t, uint32_t const * const, const uint32_t);
-LIBMTP_object_t *LIBMTP_Make_List(LIBMTP_mtpdevice_t *, uint32_t *, uint32_t, uint32_t *, uint32_t);
-LIBMTP_object_t *LIBMTP_Find_Object(LIBMTP_object_t *, const uint32_t);
-void LIBMTP_Dump_List(LIBMTP_object_t *);
-LIBMTP_object_t *LIBMTP_new_object_t(void);
-void LIBMTP_destroy_object_t(LIBMTP_object_t *, uint32_t);
 int LIBMTP_Delete_Object(LIBMTP_mtpdevice_t *, uint32_t);
-/** @} */
 
-/**
- * @}
- * @defgroup File mapping The file mapping API
- * @{
- */
-int LIBMTP_Register_Filetype(char const * const, LIBMTP_filetype_t const,
-			     uint16_t const, void const * const,
-			     void const * const, void const * const);
-int LIBMTP_Set_Filetype_Description(LIBMTP_filetype_t, char const * const);
-int LIBMTP_Set_Constructor(LIBMTP_filetype_t, void const * const);
-int LIBMTP_Set_Destructor(LIBMTP_filetype_t, void const * const);
-int LIBMTP_Set_Datafunc(LIBMTP_filetype_t, void const * const);
 /** @} */
 
-
 /* End of C++ exports */
 #ifdef __cplusplus
 }
