@@ -9,24 +9,17 @@
  * use MTP devices. Include mtp-utils.h to use any of the ptp/mtp functions.
  *
  */
+#include "libmtp.h"
+#include "libusb-glue.h"
+#include "util.h"
 #include "ptp.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <getopt.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <utime.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <sys/mman.h>
 #include <usb.h>
 
-#include "libmtp.h"
-#include "libusb-glue.h"
-#include "util.h"
-
 /* To enable debug prints, switch on this */
 //#define ENABLE_USB_BULK_DEBUG
 
@@ -61,6 +54,8 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   
   /*
    * Creative Technology
+   * Initially the Creative devices was all we supported so these are
+   * the most thoroughly tested devices.
    */
   { "Creative Zen Vision", 0x041e, 0x411f, DEVICE_FLAG_NONE },
   { "Creative Portable Media Center", 0x041e, 0x4123, DEVICE_FLAG_NONE },
@@ -183,10 +178,23 @@ static const LIBMTP_device_entry_t mtp_device_table[] = {
   /*
    * Dunlop (OEM of EGOMAN ltd?) reported by Nanomad
    * This unit is falsely detected as USB mass storage in Linux
-   * so special care is needed to blacklist the device in kernel
-   * or similar to get it properly userspace:d.
+   * prior to kernel 2.6.19 (fixed by patch from Alan Stern)
+   * so on older kernels special care is needed to remove the
+   * USB mass storage driver that erroneously binds to the device
+   * interface.
    */
-  { "Dunlop MP3 player 1GB / EGOMAN MD223AFD", 0x10d6, 0x2200, DEVICE_FLAG_UNLOAD_DRIVER}
+  { "Dunlop MP3 player 1GB / EGOMAN MD223AFD", 0x10d6, 0x2200, DEVICE_FLAG_UNLOAD_DRIVER},
+  
+  /*
+   * Microsoft
+   */
+  // Reported by Farooq Zaman
+  { "Microsoft Zune", 0x045e, 0x0710, DEVICE_FLAG_NONE }, 
+  
+  /*
+   * Sirius
+   */
+  { "Sirius Stiletto", 0x18f6, 0x0102, DEVICE_FLAG_NONE }
   
 };
 static const int mtp_device_table_size = sizeof(mtp_device_table) / sizeof(LIBMTP_device_entry_t);
@@ -555,9 +563,17 @@ static int init_ptp_usb (PTPParams* params, PTP_USB* ptp_usb, struct usb_device*
      */
     if (ptp_usb->device_flags & DEVICE_FLAG_UNLOAD_DRIVER) {
       if (usb_detach_kernel_driver_np(device_handle, dev->config->interface->altsetting->bInterfaceNumber)) {
-	perror("usb_detach_kernel_driver_np()");
+	// Totally ignore this error!
+	// perror("usb_detach_kernel_driver_np()");
       }
     }
+#endif
+#ifdef __WIN32__
+    // Only needed on Windows
+    if (usb_set_configuration(device_handle, dev->config->bConfigurationValue)) {
+      perror("usb_set_configuration()");
+      return -1;
+    }
 #endif
     if (usb_claim_interface(device_handle, dev->config->interface->altsetting->bInterfaceNumber)) {
       perror("usb_claim_interface()");
