@@ -164,26 +164,23 @@ ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t o
 	for (i=0;i<packedlen && i< PTP_MAXSTRLEN; i++) {
 		htod16a(&data[offset+i*2+1],ucs2str[i]);
 	}
+	htod16a(&data[offset+i*2+1],0x0000);
 
 	/* The returned length is in number of characters */
-	*len = (uint8_t) packedlen;
+	*len = (uint8_t) packedlen+1;
 }
 
 static inline unsigned char *
 ptp_get_packed_stringcopy(PTPParams *params, char *string, uint32_t *packed_size)
 {
-uint8_t packed[PTP_MAXSTRLEN+3], len;
+	uint8_t packed[PTP_MAXSTRLEN*2+3], len;
 	size_t plen;
 	unsigned char *retcopy = NULL;
 
 	ptp_pack_string(params, string, (unsigned char*) packed, 0, &len);
 	/* returned length is in characters, then one byte for string length */
 	plen = len*2 + 1;
-	/* Assure proper termination, two zero bytes */
-	packed[plen] = 0x00;
-	packed[plen+1] = 0x00;
-	/* Include terminator */
-	plen += 2;
+	
 	retcopy = malloc(plen);
 	if (!retcopy) {
 		*packed_size = 0;
@@ -832,6 +829,106 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr
 	return size;
 }
 
+#define MAX_MTP_PROPS 127
+static inline uint32_t
+ptp_pack_OPL (PTPParams *params, MTPPropList *proplist, unsigned char** opldataptr)
+{
+	unsigned char* opldata;
+	MTPPropList *propitr;
+	unsigned char *packedprops[MAX_MTP_PROPS];
+	uint32_t packedpropslens[MAX_MTP_PROPS];
+	uint32_t packedobjecthandles[MAX_MTP_PROPS];
+	uint16_t packedpropsids[MAX_MTP_PROPS];
+	uint16_t packedpropstypes[MAX_MTP_PROPS];
+	uint32_t totalsize = 0;
+	uint32_t bufp = 0;
+	uint32_t noitems = 0;
+	uint32_t i;
+
+	totalsize = sizeof(uint32_t); /* 4 bytes to store the number of elements */
+	propitr = proplist;
+	while (propitr != NULL && noitems < MAX_MTP_PROPS) {
+		/* Object Handle */
+		packedobjecthandles[noitems]=propitr->ObjectHandle;
+		totalsize += sizeof(uint32_t); /* Object ID */
+		/* Metadata type */
+		packedpropsids[noitems]=propitr->property;
+		totalsize += sizeof(uint16_t);
+		/* Data type */
+		packedpropstypes[noitems]= propitr->datatype;
+		totalsize += sizeof(uint16_t);
+		/* Add each property to be sent. */
+	        packedpropslens[noitems] = ptp_pack_DPV (params, &propitr->propval, &packedprops[noitems], propitr->datatype);
+		totalsize += packedpropslens[noitems];
+		noitems ++;
+		propitr = propitr->next;
+	}
+
+	/* Allocate memory for the packed property list */
+	opldata = malloc(totalsize);
+
+	htod32a(&opldata[bufp],noitems);
+	bufp += 4;
+
+	/* Copy into a nice packed list */
+	for (i = 0; i < noitems; i++) {
+		/* Object ID */
+		htod32a(&opldata[bufp],packedobjecthandles[i]);
+		bufp += sizeof(uint32_t);
+		htod16a(&opldata[bufp],packedpropsids[i]);
+		bufp += sizeof(uint16_t);
+		htod16a(&opldata[bufp],packedpropstypes[i]);
+		bufp += sizeof(uint16_t);
+		/* The copy the actual property */
+		memcpy(&opldata[bufp], packedprops[i], packedpropslens[i]);
+		bufp += packedpropslens[i];
+		free(packedprops[i]);
+	}
+	*opldataptr = opldata;
+	return totalsize;
+}
+
+static inline int
+ptp_unpack_OPL (PTPParams *params, unsigned char* data, MTPPropList **proplist, unsigned int len)
+{ 
+	uint32_t prop_count = dtoh32a(data);
+	MTPPropList *prop = NULL;
+	int offset = 0, i;
+
+	if (prop_count == 0) {
+		*proplist = NULL;
+		return 0;
+	}
+	data += sizeof(uint32_t);
+	*proplist = malloc(sizeof(MTPPropList));
+	prop = *proplist;
+	for (i = 0; i < prop_count; i++) {
+		prop->ObjectHandle = dtoh32a(data);
+		data += sizeof(uint32_t);
+		len -= sizeof(uint32_t);
+
+		prop->property = dtoh16a(data);
+		data += sizeof(uint16_t);
+		len -= sizeof(uint16_t);
+
+		prop->datatype = dtoh16a(data);
+		data += sizeof(uint16_t);
+		len -= sizeof(uint16_t);
+
+		offset = 0;
+		ptp_unpack_DPV(params, data, &offset, len, &prop->propval, prop->datatype);
+		data += offset;
+		len -= offset;
+
+		if (i != prop_count - 1) {
+			prop->next = malloc(sizeof(MTPPropList));
+			prop = prop->next;
+		} else
+			prop->next = NULL;
+	}
+	return prop_count;
+}
+
 /*
     PTP USB Event container unpack
     Copyright (c) 2003 Nikolai Kopanygin
